/****************************************************************/
/*   IMPLEMENTATION OF ADAPTIVE DIFFERENTIAL PULSE-CODE		*/
/*   MODULATION (ADPCM) TRANSCODER WITH THE			*/
/*   WE(R) DSP16 DIGITAL SIGNAL PROCESSOR			*/
/*   October, 1988 Doc# AP88-07DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1988 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*							*/
/*   dec32.s						*/
/*							*/
/* This program implements the ADPCM decoder 		*/
/* according to the CCITT standard algorithm described	*/
/* in G.721.  All the notations used here are based 	*/
/* on the CCITT 32 kb/s documents.			*/
/*							*/
/* The multiplication operations in fmult block is	*/
/* modified slightly here to increase the DSP16 	*/
/* performance with the bit precision kept.		*/
/*							*/
/* The ADPCM input is taken from parallel port #0 	*/
/* and the PCM result is written to parallel port 	*/
/* #0.  The output data is mu-law.			*/
/*							*/
/* This coder can be run in DSP16A development system.	*/
/* This coder passes all the Bellcore test vectors.	*/
/*                                                      */
/* This code file rec'd: 9/12/90 from Y.-S. Lai - AT&T	*/
/*							*/
/********************************************************/
/* Modified 9/17/90 by Peter Melsa - Tellabs Research	*/
/* Center to meet the CCITT specifications.		*/
/*							*/
/* No claims are made concerning the accuracy of this	*/
/* code beyond the fact that the code passes the 	*/
/* Bellcore test vectors.  The authors will in no way	*/
/* be held liable for any damages resulting from the	*/
/* use of this code.					*/
/*			-P. Melsa			*/
/********************************************************/

	goto sfull2

	3*nop

sfull2:
	a0=pdx0
	r0=count
	r1=0
	*r0=r1

/********************************************************/
/*							*/
/*	Program Initialization Decoder			*/
/*							*/
/********************************************************/

dinit:	auc=0xc		/* Set auc register		*/
	y=0x0		/**	RESET SECTION		*/
	r1=DYL		/* Set initial values for	*/
	*r1++=y		/* Quantizer Scale Factor	*/
	y=0x8800	/* Adaptation block.		*/
	*r1=y
	r2=DYU
	y=544
	*r2=y
	y=0x0		/* Initialize the Adaptation	*/
	r0=DDML		/* Speed Control block		*/
	*r0=y
	r1=DDMS
	*r1=y
	r3=DDAP
	*r3=y
	r0=DDQ1		/* Initialize the Adaptive	*/
	r1=DDQNT1	/* and Reconstructed Signal	*/
	y=1		/* Calculator block		*/
	do 8 {
	*r0++=y
	*r1++=y
	}
	r0=dtemp1
	*r0=y
	y=0x0
	r0=DPK1
	do 2 {
	*r0++=y
	}
	r0=DB1
	r1=DDQ1S
	do 8 {
	*r0++=y
	*r1++=y
	}
	r1=DTD		/* Initialize the Tone and	*/
	*r1=y		/* Transition Detector block.	*/

/*	sioc=0x0
	pioc=0x0
	tdms=0x0
*/

/********************************************************/
/*							*/
/*		Main Program				*/
/*							*/
/********************************************************/

loop:	y=0x10
/*
loop1:	a0=pioc
	a0&y		
	if eq goto loop1
*/
	r0=count
	a0=*r0
	a1h=a0h + 1
	*r0=a1

/* Input the ADPCM word */

	r0=DI
	a0=pdx0
	*r0=a0

adpcmd:

dpred:	c0=-7		/** AFTER DELAY PARTS aparsc.s **/
	r0=DDQNT1	/* Set the points before doing	*/
	r1=DDQ1		/* FMULT.			*/
	r2=DWB1-1
	r3=DB1
	i=-1
	j=0x40
dl0b18:	a0=*r3
	y=0xfffc
	a0=a0&y  *r2++=a1l
	if mi a0=-a0	/* a0 = |a0|			*/
	if mvs a0=a0<<16/* If a0=0x8000, set a0=0x0	*/
	a0=a0>>1
	if eq a0h=a0h+1	/* If a0=0, then a0 = a0 + 1	*/
	c1=-16		/* Set counter c1 = 16		*/
	a1=a0		/* a1 = a0			*/
	do 15 {		/* Compute exponent		*/
	ifc pl a1=a1<<1
	}
	a1=c2		/* Now c2 = -( exponent )	*/
	y=tabl1
	a1=a1+y
	pt=a1
	y=a0 x=*pt++	/* y = |DBn| and x is a pattern	*/
	a0=x		/* used to round |DBn|.		*/
	a0=a0&y  a1=*r3++
	a1=a1
	if mi a0=-a0	/* OK, a0 = two's complement	*/
	a1=*r0++	/* value of DBn with the same	*/
	y=-16		/* accuracy as standard.	*/
	a1-y
	if eq a0=a0<<1
	x=a0
	a0=c2
	y=a0
	a1=a1+y y=*r1++	/* y = DDQn			*/
	p=x*y		/* p = DBn * DDQn			*/
	a0=j
	y=-22
	a1-y		/* Compare a1 with 22 to set	*/
	if le a0=a0<<8	/* If a1 >= 22, set a0 = 0x400.	*/
	a1=a1<<1	/* a1 = a1 * 2 for the offset	*/
	y=tabl2
	a1=a1+y		/* a1 points to the table.	*/
	pt=a1
	a1=p y=a0 x=*pt++i
	p=x*y y=a0 x=*pt++
	a0=a1
	if mi a1=-a1
	a1=a1+p  p=x*y
	y=a1		/* Set y (32 bits) = a1.	*/
	a1=a1<<16
	yl=a1
	a1=p
	a1=a1&y
	a1=a1>>8	/* Rescale a1.			*/
	a1=a1>>4
	a1=a1>>1
	a0=a0		/* Check the sign bit.		*/
	if mi a1=-a1	/* If negative, then a1 = -a1.	*/
	if c0lt goto dl0b18	/* Do eight times.	*/
	*r2=a1l

/********************************************************/
/*							*/
/*			ACCUM				*/
/*							*/
/********************************************************/

	r0=DWB1		/**	 ACCUM		       **/
	a0=*r0++
	y=*r0++
	do 5 {
	a0=a0+y y=*r0++	/* Sum for partial signal	*/
	}		/* estimate			*/
	a1=a0+y  y=*r0	/* a0 = DSEZI			*/
	r0=DSEZ		/* DSEZ is 16 bits TC		*/
	a1=a1+y y=a0 x=*pt++
	a0=y
	a0=a0>>1
	*r0=a0		/* a1 = DSEI			*/
	r1=DSE
	y=a1
	a1=y
	a1=a1>>1
	r3=DDAP		/** AfTER DELAY LIMA adspcl.s  **/
	a0=*r3
	a0=a0>>1
	a0=a0>>1
	y=0x3f		/* y = ( 255 >> 2 )		*/
	yl=0xc000
	a0-y  *r1=a1	/* DSE is 16 bits TC		*/
	y=64
	if gt a0=y
	x=a0		/* DAL = x = a0			*/
	r2=DYU		/** AFTER DELAY MIX qsfadp.s   **/
	r0=DYL
	a1=*r0++
	a1l=*r0--
	a1=a1<<8
	a1=a1<<1
	a1=a1<<1
	move y=a1	/* y = (DYL >> 6)		*/
	a0=*r2		/*a0 = DYU			*/
	a0=a0-y		/* a0 = DIFM			*/
	y=a0
	p=x*y		/* p = PRODM = (DIFM * AL) >> 6.*/
	a0=p
	if mi a0=-a0
	a0=a0<<8
	a0=a0<<1
	a0=a0<<1
	move y=a0	/* y = PROD			*/
	a0=y
	y=a1
	a1=p
	if mi a0=-a0
	a1=a0+y
	y=0x1fff
	r2=DI
	a1=a1&y  a0=*r2	/* a0 = DI			*/
	r1=DY		/* DY = Standard DY +- 1 if (DYL>>6) > DYU	*/

/********************************************************/
/*							*/
/*			RECONST				*/
/*							*/
/********************************************************/

dinadq1:r3=DYSH2	/**	 RECONST  inadqu.s     **/
	y=0xf
	a0=a0^y  *r1=a1
	y=0x08
	a0&y  y=*r2
	if ne a0=y
	y=dqln
	a0=a0+y  a1=*r1	/* a0 is the point to DDQLN table*/
	pt=a0
	y=*r1  x=*pt++
	a0=x		/**	 ADDA		       **/
	a1=a1>>1
	a1=a1>>1
	y=a1		/* y = a1 = DY >> 2		*/
	a0=a0+y  *r3=a1
	y=0x7f		/**	 ANTILOG		*/
	a1=a0&y		/* a1 = DDMN			*/
	y=0x80		/* y = (1 << 7)			*/
	a1=a1+y		/* a1 = DDQT			*/
	a1=a1>>1
	a1=a1<<8
	a0=a0<<4
	if mi a1=a1<<16	/* If DDS=1, then DDQMAG = 0.	*/
	a0=a0>>16
	a0=a0<<4
	a0=a0<<1
	y=0xf
	a0=a0&y
	y=-14
	a0=a0+y
	c1=a0
	do 14 {
	if c1lt a1=a1>>1
	}
	r1=DDQ0S
	a0=*r2		/* a0 = DI			*/
	y=a1
	a1=y
	y=0x08
	a0=a0&y
	*r1=a0
	if ne a1=-a1	/* a1 = | a1 |			*/
	r1=DDQ16
	*r1=a1
dtrans1:a0=*r0++	/** AFTER DELAY TRANS totrde.s **/
	a0l=*r0--	/* a0 = DYL			*/
	a0=a0<<1
	x=a0
	a1=x		/* a1 = DYLINT			*/
	a1=-a1
	c0=a1
	a1=a0<<4
	a1=a1<<1
	y=31
	a1=a1&y		/* a1=YLFRAC=(DYL >> 10) & 31	*/
	y=32
	a1=a1+y		/* a1=THR1=(32 + YLFRAC)	*/
	do 8 {
	if c0lt a1=a1<<1
	}
	a0=x
	y=8
	a0-y		/* DYLINT > 8 ?			*/
	y=0x3e00	/* y = 31 << 9			*/
	if gt a1=y	/* Yes, DYLINT > 8.		*/
	a0=a1>>1
	y=a0
	a1=a1+y  a0=*r1	/* a1=THR2+(THR2>>1), r1=DDQ16	*/
	a1=a1>>1	/* a1 = DDQTHR			*/
	y=a1
	a0=a0
	if mi a0=-a0
	r2=DTD
	a0=a0-y  a1=*r2		/* a0 = DDQMAG - DDQTHR	*/
	if le a1=a1<<16		/* If less or equal, set*/
	r3=DTR			/* a0=0.		*/
	*r3=a1

/********************************************************/
/*							*/
/*			ADDB				*/
/*							*/
/********************************************************/

dapars1:a0=*r1	/**  BEFORE DELAY -- ADDB  aparsc.s    **/
	r0=DSE		/* r1=DDQ16	*/
	y=*r0
	a0=a0+y  		/* a0 = DSR		*/
	r0=DSR
	*r0=a0			/**	 FLOATB	       **/
	if mi a0=-a0		/* a0 = MAG = | DSR |	*/
	a1=a0<<1
	if mi goto dapars2
	a0=a0<<1
	if eq a0h=a0h+1
	c1=-16
	a1=a0
	do 15 {		/* Find the EXP from the MAG.	*/
	ifc pl a1=a1<<1
	}
dapars3:r2=DSRNT1
	a1=c2
	x=*r2		/* Check EXP = 16 ?		*/
	*r2--=a1
	y=tabl1		/* Use the table looking up to	*/
	*r2=x		/* find the pattern for retai-	*/
	a1=a1+y 	/* ning the bit precision in 	*/
	pt=a1		/* converting into floating	*/
	y=*r2 x=*pt++	/* format, and be represented	*/
	a1=x		/* in signed magnitude.		*/
	x=0
	p=x*y
	y=a1
	a0=a0&y  a1=*r0
	a1=a1
	if mi a0=-a0
	r2=DSR1		/* Update the DSR's parameters	*/
	y=*r2--
	*r2++=y
	*r2=a0
	r0=DDQ1		/**	 UPB		       **/
	r3=DB1-1		/* Set the points for cache.	*/
	i=128
	r2=DTR
	y=*r0++		/* y = DDQn			*/
	do 6 {		/* Do six times.		*/
	a0=*r1		/* a0 = DDQ			*/
	a1=a0+p *r3++=a1
	a1=a1^y  y=*r3	/* a1 = Un = DDQ ^ DDQn		*/
	a1=i		/* If a1 positive, then Un=0.	*/
	if mi a1=-a1	/* If a1 negative, then Un=1.	*/
	a0=a0
	if eq a1=a1<<16	/* a1 = UGDBn			*/
	a1=a1+y  a0=*r3	/* a1 = UGDBn + DBn		*/
	a0=a0>>8	/* a0 = DBn >> 8			*/
	y=a0
	a1=a1-y  a0=*r2	/* a1 = DBnP			*/
	a0=a0+p y=*r0++	/**	 TRIGB		       **/
	if ne a1=a1<<16	/* If =| 0, set a1=0.		*/
	}
	*r3=a1
	a0=*r1		/**	 FLOATA		       **/
	a0=a0<<1	/* a0 = DDQ, r1=DDQ16		*/
	if mi a0=-a0	/* a0 = |a0|			*/
	if eq a0h=a0h+1
	c1=-16		/* Set the initial counter to	*/
	a1=a0		/* find the EXP of a0.		*/
	do 15 {
	ifc pl a1=a1<<1
	}
	a1=c2
	r2=DDQNT0
	y=tabl1
	a1=a1+y *r2=a1
	pt=a1
	y=*r2 x=*pt++
	a1=x
	y=a1
	r0=DDQ0S
	a0=a0&y  a1=*r0	/* a0 contains the magnitude DDQ	*/
	a1=a1		/* Check the sign bit of DDQ0.	*/
	if ne a0=-a0	/* Convert it into two's comple.*/
	r2=DDQ0
	y=*r1		/**	 ADDC		       **/
	r0=DSEZ
	a1=*r0
	*r2=a0
	a0=a0<<16
	a1=a1+y  	/* a0 = DDQSEZ			*/
	if eq a0h=a0h+1	/* otherwise, set a0 = 0	*/
	r0=DSIGPK
	*r0=a0		/* Store it in SIGPK.		*/
	a0=a0<<16
	r0=DA1		/**	 UPA2		       **/
	a1=a1+p y=*r0
	if mi a0h=a0h+1	/* Find the DPK0		*/
	r2=DPK0
	a0=y *r2=a0	/* Check DA1S = 0 ?		*/
	if mi goto dlab26
	y=0x1fff	/* Yes, compare with 8191.	*/
	a0-y
	if gt a0=y	/* If a0 > 8191, set a0 = 8191	*/
	a0=a0>>16
	a0=a0<<1
	a0=a0<<1	/* a0 = a0 >> 14		*/
	goto dlab28
dlab26:	y=0x0		/* No, compare with 57345	*/
	yl=*r0
	a0=y
	yl=0xe000
	a1=a0-y		/* DA1 - 57344 >=0 ?		*/
	a0=a0<<1
	a0=a0<<1
	y=0x1
	yl=0xffff
	a0=a0&y		/* Yes, a0 = DA1 << 2		*/
	yl=0x8004
	a1=a1
	if le a0=y	/* No, a0 = 24577 << 2		*/
dlab28:	y=*r2
	r1=DPK1
	a1=y y=*r1 x=*pt++
	a1=a1^y		/* a1 = DPK0 ^ DPK1 = DPKS1	*/
	if eq a0=-a0	/* a0 = FA			*/
	y=*r2
	r1=DPK2
	a1=y y=*r1 x=*pt++
	a1=a1^y		/* a1 = DPK0 ^ DPK2 = DPKS2	*/
	y=0x1		/* Set y = 114688 if a1 < 0	*/
	yl=0xc000
	if ne goto dlab31
	y=0x0		/* otherwise, set y = 16384	*/
	yl=0x4000
dlab31:	r0=DSIGPK
	a0=a0+y  a1=*r0	/* a0 = UGA2B			*/
	a0=a0>>1
	a0=a0<<16
	a0=a0>>4
	a0=a0>>1
	a0=a0>>1	/* a0 = a0 << 9			*/
	y=a0
	a0=y
	y=0x0
	a1=a1		/* Check DSIGPK = 0 ?		*/
	if ne a0=y	/* If not, set a0 = 0.		*/
	r0=DA2		/* a0 = UGA2			*/
	y=*r0
	a0=a0+y		/* a0 = UGA2 + DA2		*/
	a1=y		/* a1 = DA2			*/
	a1=a1>>8
	a1=a1<<1	/* a1 = DA2 >> 7			*/
	y=a1
	a0=a0-y		/* a0 = DA2T			*/
	a1=a0		/**	 LIMC		       **/
	if mi a1=-a1	/* a1 = |a1|			*/
	y=0x3000	/* Set y = 12288		*/
	a1-y		/* |a1| > 12288 ?		*/
	if pl a1=y	/* Yes, set |a1| = 12288	*/
	r1=DTR		/**	 TRIGB		       **/
	a0=a0+p y=*r1
	if mi a1=-a1	/* a1 = DA2P			*/
	r0=DA2P
	*r0=a1
	a0=y
	if ne a1=a1<<16	/* If DTR =| 0, set a1 = 0.	*/
	r1=DA2
	*r1=a1
	r1=DPK1		/**	 UPA1		       **/
	r0=DPK0
	a1=*r1++
	y=*r0
	a1=a1^y  *r1--=a1	/* a1 = DPK0 ^ DPK1 = DPKS	*/
	*r1=y
	y=192
	move a0=y	/* Set a0 = 192 if DPKS = 0	*/
	if ne a0=-a0	/* otherwise, set a0 = 65344	*/
	r0=DSIGPK
	a1=*r0
	a1=a1
	if ne a0=a0<<16	/* a0=UGA1			*/
	r0=DA1
	y=*r0		/* y = DA1			*/
	a0=a0+y		/* a0 = UGA1 + DA1		*/
	a1=y		/* a1 = DA1			*/
	a1=a1>>8	/* a1 = DA1 >> 8		*/
	y=a1
	a0=a0-y		/* a0 = UGA1 + DA1 - (DA1 >> 8)	*/
	r2=DA1T
	y=0x3c00	/**	 LIMD		       **/
	r1=DA2P		/* Set y = 15360		*/
	a1=*r1
	a1=a1-y  *r2=a0	/* a1 = DA2P - 15360, a0 = DA1T	*/
	a1=-a1
	y=a1		/* y = A1UP			*/
	a1=a0
	if mi a1=-a1	/* a1 = | DA1T |		*/
	a1-y		/* | DA1T | - A1UP		*/
	if pl a1=y
	a0=a0
	if mi a1=-a1	/* a1 = DA1P			*/
	r0=DTR		/**	 TRIGB		       **/
	a0=*r0
	a0=a0		/* Check DTR = 0 ?		*/
	if ne a1=a1<<16	/* If not, then set a1 = 0	*/
	r0=DA1
	*r0=a1		/* otherwise, keep a1.		*/
	r0=DDQ0
	y=*r0++
	*r0zp:y		/* Update the DDQn parameters	*/
			/* the next input data.		*/
	*r0zp:y
	*r0zp:y
	*r0zp:y
	*r0zp:y
	*r0zp:y
	r0=DDQNT0
	y=*r0++
	*r0zp:y		/* Update the parameters for	*/
			/* the next input data.		*/
	*r0zp:y
	*r0zp:y
	*r0zp:y
	*r0zp:y
	*r0zp:y

/********************************************************/
/*							*/
/*			TONE				*/
/*							*/
/********************************************************/

dtotrd1:r0=DA2P		/**	 TONE  totrde.s	       **/
	a0=*r0
	y=0x2e00	/* y = 11776			*/
	a1=a1<<16
	a0=a0+y		/* a2(k) + 0.71875		*/
	if mi a1h=a1h+1	/* a1 = DTDP			*/
	r0=DTDP
	*r0=a1
	r3=DTR		/**	 TRIGB		       **/
	a0=*r3
	a0=a0		/* Check DTR = 0 ?		*/
	if ne a1=a1<<16	/* If not, set a1 = 0		*/
	r1=DTD		/* otherwise, keep al.		*/
	*r1=a1
dadspc1:r0=DI		/**	 FUNCTF  adspcl.s      **/
	a0=*r0		/* a0 = DI			*/
	y=FDI
	a0=a0+y		/* Use the table looking up to	*/
	r2=DDML		/* find the FI (scaled) value.	*/
	pt=a0
	y=*r2 x=*pt++
	a0=x		/* a0 = FI			*/
	a1=a0<<1	/**	 FILTB		       **/
	a1=a1<<1	/* a1 = FI << 11		*/
	a1=a1-y		/* a1 = (FI<<11) - DDML = DIF	*/
	a1=a1<<1
	a1=a1>>8	/* a1 = DIF >> 7 = DIFSX	*/
	a1=a1+y		/* a1 = DIFSX + DDML		*/
	y=0x3fff	/* Set y = 16383		*/
	r1=DDMS
	a1=a1&y y=*r1
	a0=a0-y	 *r2=a1 	/**	 FILTA	       **/
	a0=a0>>1
	a0=a0<<4
	a0=a0>>8	/* a0 = DIFSX = DIF >> 5	*/
	a0=a0+y		/* a0 = DIFSX + DDMS		*/
	y=0xfff
	a0=a0&y y=*r2
	*r1=a0		/* a0 = DDMSP			*/
	a0=a0<<1	/**	 SUBTC		       **/
	a0=a0<<1	/* a0 = DDMSP << 2		*/
	a0=a0-y  a1=*r2	/* a0 = (DDMSP<<2) - DDMLP = DIF	*/
	if mi a0=-a0	/* a0 = | DIF | = DIFM		*/
	a1=a1>>4
	a1=a1<<1	/* a1 = DTHR = DDMLP >> 3	*/
	move y=a1
	r3=DY
	a0=a0-y  a1=*r3		/* a0 = DIFM - DTHR	*/
	r3=DTDP
	a0=*r3
	y=1
	if pl a0=y
	y=1536		/* Set y = 1536			*/
	a1=a1-y		/* a1 = DY - 1536		*/
	y=1
	if mi a0=y	/* a0 = AX			*/
	a0=a0<<8	/**	 FILTC		       **/
	a0=a0<<1	/* a0 = AX << 9			*/
	r3=DDAP
	y=*r3		/* y = DDAP			*/
	a0=a0-y		/* a0 = (AX<<9) - DDAP = DIF	*/
	a0=a0>>4	/* a0 = DIFSX			*/
	a0=a0+y		/* a0 = DIFSX + DDAP		*/
	y=0x3ff
	r2=DTR		/**	 TRIGA		       **/
	a0=a0&y y=*r2	/* a0 = DDAPP			*/
	a1=y *r2
	y=0x100		/* Set y = 256			*/
	if ne a0=y
	*r3=a0		/* a0 = DDAPR			*/

/********************************************************/
/*							*/
/*			FUNCTW				*/
/*							*/
/********************************************************/

dqsfad1:r0=DI		/**	 FUNCTW  qsfadp.s      **/
	a0=*r0
	y=WDI
	a0=a0+y		/* a0 points to WI		*/
	pt=a0
	y=*r0 x=*pt++	/* x = WI			*/
	a0=x		/**	 FILTD		       **/
	a0=a0>>8
	a0=a0>>4
	a0=a0<<1	/* a0l = WI << 5		*/
	r2=DY
	y=0x0
	yl=*r2		/* y = DY			*/
	a0=a0-y		/* a0 = DIF			*/
	a0=a0>>4
	a0=a0>>1	/* a0 = DIFSX = DIF >> 5	*/
	a0=a0+y		/* a0 = DY + DIFSX		*/
	a0=a0<<16
	y=0x1fff	/* Set y = 8191			*/
	a0=a0&y		/* a0 = DYUT			*/
	y=544		/**	 LIMB		       **/
	a0-y		/* If negative, GELL = 1	*/
	if mi a0=y
	y=5120		/* Set y = 5120			*/
	a0-y		/* If positive, GEUL = 0	*/
	if pl a0=y	/* a0 = DYUP			*/
	r3=DYU
	*r3=a0		/* Store it.			*/
	y=0
	yl=a0
	r1=DYL		/**	 FILTE		       **/
	a0=*r1++
	a0l=*r1--	/* a0 = DYL			*/
	a0=-a0		/* a0 = -DYL			*/
	a0=a0>>4
	a0=a0>>1
	a0=a0>>1	/* a0 = ( -DYL ) >> 6		*/
	a0=a0+y		/* a0 = DIF = DIFSX		*/
	y=*r1++
	yl=*r1--	/* y = DYL			*/
	a0=a0+y		/* a0 = DYL + DIFSX		*/
	y=0x7
	yl=0xffff
	a0=a0&y		/* a0 = DYLP			*/
	*r1++=a0
	*r1=a0l		/* Store it.			*/

/********************************************************/
/*							*/
/*			LINEAR				*/
/*							*/
/********************************************************/

/*	Linear output is stored in DSR			*/


comprs: r3=dtemp             /**         COMPRESS  comprs.s        **/
        r0=DSR               /* Convert from uniform PCM to         */
        a0=*r0              /* u-law PCM.                          */
        a1=a0>>8
        y=0x80
        a1=a1&y             /* Extract the sign bit.               */
        *r3=a1              /* Store it.                           */
        if ne a0=-a0        /* a0 = IMAG                           */
        y=8158
        a1=a0-y             /* Beyond the border ?                 */
        if gt a0=y          /* Yes, set the maximal value          */
        y=33
        a0=a0+y             /* a0 = a0 + 33                        */
        c1=-10              /* Set the initial counter             */
        do 10 {
        ifc pl a0=a0<<1     /* Find the abc value.                 */
        }
        a0=a0<<1
        a0=a0>>4
        a0=a0>>8
        y=0xf
        a0=a0&y             /* a0 is the xyzw value.               */
        a1=c2
        a1=-a1              /* a1 = abc                            */
        a1=a1<<4
        y=a1
        a0=a0+y  y=*r3      /* y = the sign bit                    */
        a0=a0+y             /* a0 = the before bit inverted        */
        y=0xff
        a0=a0^y             /* a0 = u-law                          */
        r1=DS
        *r1=a0              /* Store it.                           */

/*******************************************************************/
/*                                                                 */
/* For the non-standard CCITT ADPCM mode, the program from         */
/* the ipcdsc label, (EXPAND sub-program), to the line             */
/* before the End of Program, (goto loop), will be deleted.        */
/* The output will be stored in RAM location S.                    */
/*                                                                 */
/*******************************************************************/

/*******************************************************************/
/*                                                                 */
/*                        EXPAND                                   */
/*                                                                 */
/*******************************************************************/

ipcdsc: r1=DS                /**         EXPAND  ipcmv.s           **/
        a0=*r1
        y=0xff              /* Invert bits.                        */
        a0=a0^y             /* This program only considers         */
        a1=a0>>4            /* the input as u-law.                 */
        y=0x7
        a1=a1&y             /* a1 = L = abc                        */
        a1=-a1
        c0=a1
        y=0xf
        a1=a0&y             /* a1 = V = xyzw                       */
        a1=a1<<1
        y=0x21
        a1=a1|y
        do 7 {
        if c0lt a1=a1<<1
        }
        y=33
        a1=a1-y
        y=0x80              /* Check the sign bit.                 */
        r0=DSE
        a0=a0&y y=*r0
        if ne a1=-a1        /* If negative, then a1 = -a1.         */
        r1=DD                /**         SUBTA                     **/
        a1=a1-y
        *r1=a1

/******************************************************************/
/*                                                                */
/*                        LOG                                     */
/*                                                                */
/******************************************************************/

dadaqu1: r0=DD                /**         LOG  adaqua.s            **/
        y=*r0
        a0=y  *r0
        if mi a0=-a0        /* Convert D from 2's complement.     */
        c1=-15              /* to signed magnitude                */
        a1=a0
        do 15 {
        ifc pl a1=a1<<1     /* Compute exponent                   */
        }
        y=0x7f00
        a0=a1&y
        a0=a0>>8
        a1=c2
        y=a0
        a1=-a1
        a1=a1<<8
        a1=a1>>1            /* a1 = EXP << 7                      */
        r0=DYSH2
        a1=a1+y  y=*r0      /* y = Y >> 2                         */
        a1=a1-y             /**         SUBTB                    **/
        y=4095
        a1=a1&y
        r0=DDLN              /* DLN = (DL + 4096 - y) & 4095       */
        *r0=a1

/******************************************************************/
/*                                                                */
/*                        SYNC                                    */
/*                                                                */
/******************************************************************/

sync:   r3=dtemp1            /**         SYNC  sync.s             **/
        r0=DDLN
        a1=*r0              /* a1 = DLNX                          */
        a1=a1>>16           /* a1h = 0x0, a1l = DLNX              */
        r1=0                /* Set counter to find the table      */
        pt=dln              /* offset.                            */
        y=*r3  x=*pt++
ldb3:   p=x*y  y=*r3  x=*pt++    /* Find the table offset         */
        a0=a1-p  *r1++
        if gt goto ldb3
        r0=DD
        y=*r0
        a0=y  *r0           /* Check positive or negative ?       */
        y=posx-1
        if pl goto ldb5
        y=negx-1
ldb5:   a0=r1
        a0=a0+y             /* a0 points to the desired DLNX      */
        pt=a0               /* in the table.                      */
        y=*r0  x=*pt++
        r1=DS
        a0=*r1              /* a0 = S                             */
        y=0xff
        a0=a0^y             /* Bit inverted                       */
        r0=DI
        a1=*r0              /* a1 = I                             */
        y=DIM
        a1=a1+y             /* a1 points to the desired IM        */
        pt=a1               /* in the table IM.                   */
        a1=x
        y=a1  x=*pt++
        a1=x                /* a1 = IM                            */
        a1=a1-y             /* Check IM-ID ?                      */
        if eq goto ldb17    /* If zero, SD = SP.                  */
        if pl goto ldb11    /* If positive, SD = SP+              */
        y=0x80              /* If negative, SD = SP-              */
        a1=a0&y
        if eq goto ldb8
        y=255
        a1=a0-y
        if ne a0h=a0h+1
        goto ldb17
ldb8:   y=1                 /* The SP- represents the next        */
        a0=a0-y             /* more negative PCM output           */
        y=129               /* level.                             */
        if mi a0=y
        goto ldb17
ldb11:  y=0x80
        a0&y
        if eq goto ldb14
        y=1                  /* The SP+ represents the next       */
        a0=a0-y              /* more positive PCM output          */
        y=127                /* level.                            */
        a0-y
        y=1
        if eq a0=y
        goto ldb17
ldb14:  y=127
        a1=a0-y
        if ne a0h=a0h+1
ldb17:  y=0xff               /* Do bit inversion.                 */
        a0=a0^y
        r1=DSD
        *r1=a0               /* Store it.                         */
	pdx0 = a0	/* Output PCM value */
	goto loop	/* Go back to execute the next	*/
			/* sampling data.		*/
dapars2:c2=-16
	goto dapars3

stop:	nop
	goto stop

/********************************************************/
/*							*/
/*		End of Program				*/
/*							*/
/********************************************************/

posx:	int	0x9
	int	0xa
	int	0xb
	int	0xc
	int	0xd
	int	0xe
	int	0xf
	int	0x7
	int	0x9
negx:	int	0x6
	int	0x5
	int	0x4
	int	0x3
	int	0x2
	int	0x1
	int	0x0
	int	0x7
	int	0x6
WI:
WDI:	int	0xfff4		/* -12 = 4084 for 12 bits	*/
	int	18
	int	41
	int	64
	int	112
	int	198
	int	355
	int	1122
	int	1122
	int	355
	int	198
	int	112
	int	64
	int	41
	int	18
	int	0xfff4
FI:
FDI:	int	0
	int	0
	int	0
	int	0x200
	int	0x200
	int	0x200
	int	0x600
	int	0xe00
	int	0xe00
	int	0x600
	int	0x200
	int	0x200
	int	0x200
	int	0
	int	0
	int	0
pos:	int	0x01
	int	0x02
	int	0x03
	int	0x04
	int	0x05
	int	0x06
	int	0x07
	int	0x0f
	int	0x01
neg:	int	0x0e
	int	0x0d
	int	0x0c
	int	0x0b
	int	0x0a
	int	0x09
	int	0x08
	int	0x0f
	int	0x0e
dln:	int	79
	int	177
	int	245
	int	299
	int	348
	int	399
	int	2047
	int	3971
	int	4095
dqln:	int	2048
	int	4
	int	135
	int	213
	int	273
	int	323
	int	373
	int	425
IM:	int	0x8
	int	0x9
	int	0xa
	int	0xb
	int	0xc
	int	0xd
	int	0xe
	int	0xf
	int	0x0
	int	0x1
	int	0x2
	int	0x3
	int	0x4
	int	0x5
	int	0x6
	int	0x7

	int	0x7e00
	int	0x7e00
	int	0x3f00
	int	0x1f80
	int	0xfc0
	int	0x7e0
	int	0x3f0
	int	0x1f8
	int	0xfc
	int	0x7e
	int	0x3f
	int	0x3f
	int	0x3f
	int	0x3f
	int	0x3f
	int	0x3f
tabl1:	int	0x0

	int	0x3f00
	int	0x300
	int	0x3f80
	int	0x180
	int	0x7fc0
	int	0xc0
	int	0x7fe0
	int	0x60
	int	0x7ff0
	int	0x30
	int	0x7ff8
	int	0x18
	int	0x7ffc
	int	0xc
	int	0x7ffe
	int	0x6
	int	0x7fff
	int	0x3
	int	0x7f80
	int	0x180
	int	0x7fc0
	int	0xc0
	int	0x7fe0
	int	0x60
	int	0x7ff0
	int	0x30
	int	0x7ff8
	int	0x18
	int	0x7ffc
	int	0xc
	int	0x7ffe
	int	0x6
	int	0x7fff
	int	0x3
	27*int	0x0
tabl2:	int	0x0

DIM:	int 0x8
	int 0x9
	int 0xa
	int 0xb
	int 0xc
	int 0xd
	int 0xe
	int 0xf
	int 0x0
	int 0x1
	int 0x2
	int 0x3
	int 0x4
	int 0x5
	int 0x6
	int 0x7

.ram
DSD:	int
DDLN:	int
DS:	int
DSL:	int
DD:	int
dtemp:	int
dtemp1:	int
DY:	int
DYSH2:	int
DR:	int
DYL:	int
	int
DYU:	int
DTDP:	int
DDAP:	int
DDML:	int
DDMS:	int
DDQ0:	int
DDQ1:	int
DDQ2:	int
DDQ3:	int
DDQ4:	int
DDQ5:	int
DDQ6:	int
DSR2:	int
DSR1:	int
	int
DB1:	int
DB2:	int
DB3:	int
DB4:	int
DB5:	int
DB6:	int
DA2:	int
DA1:	int
	int
DWB1:	int
DWB2:	int
DWB3:	int
DWB4:	int
DWB5:	int
DWB6:	int
DWA2:	int
DWA1:	int
DPK1:	int
DPK2:	int
DPK0:	int
DA2P:	int
DSE:	int
DSEZ:	int
DDQ16:	int		/* The TC of DDQ		*/
DTR:	int
DA1T:	int
DSIGPK:	int
DTD:	int
DDQ0S:	int
DDQ1S:	int
DDQ2S:	int
DDQ3S:	int
DDQ4S:	int
DDQ5S:	int
DDQ6S:	int
DSR2S:	int
DSR1S:	int
DDQNT0:	int
DDQNT1:	int
DDQNT2:	int
DDQNT3:	int
DDQNT4:	int
DDQNT5:	int
DDQNT6:	int
DSRNT2:	int
DSRNT1:	int

DSR:	int
DI:	int
count:	int
scratch:	int
.endram


