/****************************************************************/
/*   IMPLEMENTATION OF ADAPTIVE DIFFERENTIAL PULSE-CODE		*/
/*   MODULATION (ADPCM) TRANSCODER WITH THE			*/
/*   WE(R) DSP16 DIGITAL SIGNAL PROCESSOR			*/
/*   October, 1988 Doc# AP88-07DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1988 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*								*/
/*   enc32org.s							*/
/*								*/
/* This	program	ADPCM encoder is coded according to the	CCITT	    */
/* standard algorithm described	in G.721. All the notations used    */
/* here	are based on the CCITT documents and the program only	    */
/* operates in u-law. The multiplication operations in the FMULT    */
/* block are modified slightly to increase the DSP16 performance    */
/* with	the bit	precision kept.	 The code in the FMULT block is	    */
/* fully compatible with the CCITT G.721 standard FMULT	code.	    */
/* The input data is assumed in	RAM location SD	before program	    */
/* execution and the result is stored in RAM location I.	    */
/*								    */
/********************************************************************/

/********************************************************************/
/*								    */
/*	Program	Initialization					    */
/*								    */
/********************************************************************/

adpcme:	auc=0xc		/* Set auc register			    */
	y=0x0		/**	    RESET SECTION		   **/
	r1=YL		/* Set initial values for		    */
	*r1++=y		/* Quantizer Scale Factor		    */
	y=0x8800	/* Adaptation block.			    */
	*r1=y
	r2=YU
	y=544
	*r2=y
	y=0x0		/* Initialize the Adaptation		    */
	r0=DML		/* Speed Control block.			    */
	*r0=y
	r1=DMS
	*r1=y
	r3=AP
	*r3=y
	r0=DQ1		/* Initialize the Adaptive		    */
	r1=DQCNT1	/* and Reconstructed Signal		    */
	y=1		/* Calculator block.			    */
	do 8 {
	*r0++=y
	*r1++=y
	}
	r0=temp1
	*r0=y
	y=0x0
	r0=PK1
	do 2 {
	*r0++=y
	}
	r0=B1
	r1=DQ1S
	do 8 {
	*r0++=y
	*r1++=y
	}
	r1=TD		/* Initialize the Tone and		    */
	*r1=y		/* Transition Detector block.		    */

/********************************************************************/
/*								    */
/*		  Main Program					    */
/*								    */
/********************************************************************/

/********************************************************************/
/*								    */
/*		  FMULT						    */
/*								    */
/********************************************************************/

loop:
pred:	c0=-7		    /**	AFTER DELAY PARTS aparsc.s	   **/
	r0=DQCNT1	    /* Set the points before doing	    */
	r1=DQ1		    /* FMULT.				    */
	r2=WB1-1
	r3=B1
l0b18:	a0=*r3
	y=0xfffc
	a0=a0&y	 *r2++=a1l
	if mi a0=-a0	    /* a0 = |a0|			    */
	a0=a0>>1
	if eq a0h=a0h+1	    /* If a0 = 0, then a0 = a0 + 1	    */
	c1=-16		    /* Set counter c1 =	16		    */
	a1=a0		    /* a1 = a0				    */
	do 15 {		    /* Compute exponent			    */
	ifc pl a1=a1<<1
	}
	a1=c2		    /* Now c2 =	-( exponent )		    */
	a1=-a1		    /* a1 = exponent part		    */
	j=a1
	y=tabl1
	a1=a1+y
	pt=a1
	y=a0 x=*pt++	    /* y = |Bn|	and x is a pattern	    */
	a0=x		    /* used to round |Bn|.		    */
	a0=a0&y	 a1=*r3++
	a1=a1
	if mi a0=-a0	    /* OK, a0 =	2's complement		    */
	a1=*r0++	    /* value of	Bn with	the same	    */
	y=16		    /* accuracy	as standard.		    */
	a1-y
	if pl a0=a0<<1
	x=a0
	a0=j
	y=a0
	a1=a1+y	 y=*r1++    /* y = DQn				    */
	p=x*y		    /* p = Bn *	DQn			    */
	y=22
	a1-y		    /* Compare a1 with 22 to set	    */
	y=0x40		    /* the right pattern with the	    */
	move a0=y	    /* table.				    */
	if pl a0=a0<<8	    /* If a1 >=	22, set	a0 = 0x400.	    */
	a1=a1<<1	    /* a1 = a1 * 2 for the offset	    */
	y=tabl2
	a1=a1+y		    /* a1 points to the	table.		    */
	pt=a1
	a1=p y=a0 x=*pt++
	p=x*y y=a0 x=*pt++
	a0=a1
	if mi a1=-a1
	a1=a1+p	 p=x*y
	y=a1		    /* Set y (32 bits) = a1.		    */
	a1=a1<<16
	yl=a1
	a1=p
	a1=a1&y
	a1=a1>>8	    /* Rescale a1.			    */
	a1=a1>>4
	a1=a1>>1
	a0=a0		    /* Check the sign bit.		    */
	if mi a1=-a1	    /* If negative, then a1 = -a1.	    */
	if c0lt	goto l0b18  /* Do eight	times.			    */
	*r2++=a1l

/********************************************************************/
/*								    */
/*			  ACCUM					    */
/*								    */
/********************************************************************/

	r0=WB1		    /**		ACCUM			   **/
	a0=*r0++
	y=*r0++
	do 5 {
	a0=a0+y	y=*r0++	    /* Sum for partial signal		    */
	}		    /* estimate				    */
	a1=a0+y	 y=*r0	    /* a0 = SEZI			    */
	a1=a1+y		    /* a1 = SEI				    */
	r0=SEZ		    /* SEZ is 16 bits TC		    */
	a0=a0>>1
	*r0=a0
	r1=SE
	a1=a1>>1
	r3=AP		    /**	AFTER DELAY LIMA adspcl.s	   **/

/********************************************************************/
/*								    */
/* For the non-standard	CCITT ADPCM mode, the program from	    */
/* the label, loop (Main program), to the previous line		    */
/* (r3=AP) replaced by the following program segment.		    */
/*								    */
/* loop:							    */
/* pred:  r1=DQ1	       ; aparsc				    */
/*	  r2=B1		       ; Set the points	before		    */
/*	  y=0		       ; doing FMULT			    */
/*	  a1=y	x=*r2++	       ; Clear a1			    */
/*	  do 6 {						    */
/*	  y=*r1++						    */
/*	  p=x*y		       ; p = DQn * Bn			    */
/*	  a0=p	x=*r2++	       ; a0 = DQn * Bn			    */
/*	  a0=a0>>1						    */
/*	  a0=a0<<4	       ; Scaling			    */
/*	  y=a0							    */
/*	  a1=a1+y						    */
/*	  }							    */
/*	  a0=a1>>1	       ; a0 = SEZ			    */
/*	  r3=SEZ						    */
/*	  *r3=a0	       ; Store it			    */
/*	  redo 2						    */
/*	  a1=a1>>1	       ; a1 = SE			    */
/*	  r1=SE							    */
/*	  r3=AP		       ; adspcl.s			    */
/*								    */
/********************************************************************/

	a0=*r3
	do 2 {
	a0=a0>>1
	}
	y=0x3f		    /* y = ( 255 >> 2 )			    */
	yl=0xc00
	a0-y  *r1=a1	    /* SE is 16	bits TC			    */
	y=64
	if gt a0=y
	r2=AL
	*r2=a0
	r2=YU		    /**	AFTER DELAY  MIX  qsfadp.s	   **/
	r1=YL
	a1=*r1++
	a1l=*r1--
	a1=a1<<8
	a1=a1<<1
	a1=a1<<1
	move y=a1	    /* y = (YL >> 6)			    */
	a0=*r2
	a0=a0-y		    /* a0 = DIFM			    */
	r0=AL
	x=*r0
	y=a0
	p=x*y		    /* p = PRODM = (DIFM * AL) >> 6.	    */
	a0=p
	a0=a0<<8
	a0=a0<<1
	a0=a0<<1
	move y=a0	    /* y = PROD				    */
	a1=a1+y
	y=0x1fff
	a1=a1&y
	r2=Y		    /* Y = Standard Y +- 1 if (YL>>6) >	YU  */
	*r2=a1

/********************************************************************/
/*								    */
/*			  EXPAND				    */
/*								    */
/********************************************************************/

ipcdsc:	r1=S		    /**		EXPAND	ipcmv.s		   **/
	a0=*r1
	y=0xff		    /* Invert bits			    */
	a0=a0^y		    /* This program only considers	    */
	a1=a0>>4	    /* the input as u-law.		    */
	y=0x7
	a1=a1&y		    /* a1 = L =	abc			    */
	a1=-a1
	c0=a1
	y=0xf
	a1=a0&y		    /* a1 = V =	xyzw			    */
	a1=a1<<1
	y=0x21
	a1=a1|y
	do 7 {
	if c0lt	a1=a1<<1
	}
	y=33
	a1=a1-y
	y=0x80		    /* Check the sign bit.		    */
	r0=SE
	a0=a0&y	y=*r0
	if ne a1=-a1	    /* If negative, then a1 = -a1.	    */
	r1=D		    /**		SUBTA			   **/
	a1=a1-y
	*r1=a1

/********************************************************************/
/*								    */
/*			  LOG					    */
/*								    */
/********************************************************************/

adaqu1:	r0=D		    /**		LOG  adaqua.s		   **/
	y=*r0
	a0=y  *r0
	if mi a0=-a0	    /* Convert D from 2's complement	    */
	c1=-15		    /* to signed magnitude		    */
	a1=a0
	do 15 {
	ifc pl a1=a1<<1	    /* Compute exponent			    */
	}
	y=0x7f00
	a0=a1&y
	a0=a0>>8
	a1=c2
	y=a0
	a1=-a1
	a1=a1<<8
	a1=a1>>1	    /* a1 = EXP	<< 7			    */
	r0=Y
	a1=a1+y	 a0=*r0	    /* a1 = (EXP << 7) + MANT		    */
	a0=a0>>1
	a0=a0>>1	    /* a0 = Y >> 2			    */
	y=a0
	r0=YSH2
	a1=a1-y	 *r0=a0	    /**		SUBTB			   **/
	y=4095
	a1=a1&y
	r0=DLN		    /* DLN = (DL + 4096	- y) & 4095	    */
	*r0=a1
quan:	r0=DLN		    /**		QUAN  adaqua.s		   **/
	a1=*r0
	a1=a1>>16
	r3=temp1
	r1=0
	pt=dln
	y=*r3  x=*pt++	    /* Compare a1 with DLN to find	    */
l6b1:	p=x*y  y=*r3  x=*pt++	 /*  the point to I.		    */
	a0=a1-p	 *r1++
	if gt goto l6b1
	r0=D
	y=*r0
	a0=y  *r0	    /* Check the sign bit.		    */
	y=neg-1
	if mi goto l6b3
	y=pos-1
l6b3:	a0=r1
	a0=a0+y
	pt=a0		    /* a0 points to I.			    */
	y=*r0  x=*pt++
	r0=I
	*r0=x

/********************************************************************/
/*								    */
/*			  RECONST				    */
/*								    */
/********************************************************************/

inadq1:	r1=YSH2		    /**	    RECONST  inadqu.s		   **/
	r2=I
	a0=*r2
	y=0xf
	a0=a0^y
	y=0x08
	a0&y y=*r2
	if ne a0=y
	y=dqln
	a0=a0+y		    /* a0 is the point to DQLN table	    */
	pt=a0
	y=*r1  x=*pt++
	a0=x		    /**		ADDA			   **/
	a0=a0+y		    /* DQL = (DQLN + (Y>>2)) & 4095	    */
	y=0x7f		    /**		ANTILOG			   **/
	a1=a0&y		    /* a1 = DMN				    */
	y=0x80		    /* y = (1<<7)			    */
	a1=a1+y		    /* a1 = DQT				    */
	a1=a1>>1
	a1=a1<<8
	y=0
	a0=a0<<4
	if mi a1=y	    /* If DS=1,	then DQMAG = 0.		    */
	a0=a0>>16
	a0=a0<<4
	a0=a0<<1
	y=0xf
	a0=a0&y
	y=-14
	a0=a0+y
	c1=a0
	do 14 {
	if c1lt	a1=a1>>1
	}
	r0=DQ0S
	a0=*r2		    /* a0 = I				    */
	y=0x08
	a0=a0&y
	a0=a0<<1	    /* Check DQS. If positive, then	    */
	a0=a0>>4	    /* set DQ0S=0, otherwise DQ0S=1.	    */
	*r0=a0
	if ne a1=-a1	    /* a1 = | a1 |			    */
	r0=DQ16
	*r0=a1
trans1:	r0=YL		    /**	AFTER DELAY TRANS totrde.s	   **/
	a0=*r0++
	a0l=*r0--	    /* a0 = YL				    */
	a0=a0<<1
	x=a0
	a1=x		    /* a1 = YLINT			    */
	a1=-a1
	c0=a1
	a1=a0<<4
	a1=a1<<1
	y=31
	a1=a1&y		    /* a1=YLFRAC=(YL >>	10) & 31	    */
	y=32
	a1=a1+y		    /* a1=THR1=(32 + YLFRAC)		    */
	do 8 {
	if c0lt	a1=a1<<1
	}
	a0=x
	y=8
	a0-y		    /* YLINT > 8 ?			    */
	y=0x3e00	    /* y = 31 << 9			    */
	if gt a1=y	    /* Yes, YLINT > 8.			    */
	a0=a1>>1
	y=a0
	a1=a1+y		    /* a1 = THR2 + (THR2 >> 1)		    */
	a1=a1>>1	    /* a1 = DQTHR			    */
	y=a1
	r1=DQ16
	a1=*r1
	a1=a1
	if mi a1=-a1
	r2=TD
	a1=a1-y	 a0=*r2	    /* a1 = DQMAG - DQTHR		    */
	if le a0=a0<<16	    /* If less or equal, set		    */
	r3=TR		    /* a0=0.				    */
	*r3=a0

/********************************************************************/
/*								    */
/*			  ADDB					    */
/*								    */
/********************************************************************/

apars1:	r0=DQ16		    /**	 BEFORE	DELAY -- ADDB  aparsc.s	   **/
	a0=*r0
	r0=SE
	r3=SR1S
	y=*r0
	a0=a0+y	 x=*r3--    /* a0 = SR				    */
	r0=SR
	*r0=a0
	y=0		    /* Set a1 =	SR sign	bit		    */
	move a1=y
	if mi a1h=a1h+1	    /**		FLOATB			   **/
	a0=a0
	if mi a0=-a0	    /* a0 = MAG	= | SR |		    */
	*r3++=x
	*r3=a1		    /* Update the sign bit storage.	    */
	a0=a0<<1
	if eq a0h=a0h+1
	c1=-16
	c2=-16
	a1=a0
	do 15 {		    /* Find the	EXP from the MAG.	    */
	ifc pl a1=a1<<1
	}
	r2=SRCNT1
	a1=c2
	a1=-a1
	y=16
	a1-y  x=*r2	    /* Check EXP = 16 ?			    */
	*r2--=a1
	if eq a0=a0>>1	    /* Yes, shift it back.		    */
	y=tabl1		    /* Use the table looking up	to	    */
	*r2=x		    /* find the	pattern	for retaining	    */
	a1=a1+y		    /* the bit precision in		    */
	pt=a1		    /* converting into floating		    */
	y=*r2 x=*pt++	    /* format, and be represented	    */
	a1=x		    /* in signed magnitude.		    */
	y=a1
	a0=a0&y	 a1=*r3
	a1=a1
	if ne a0=-a0
	r2=SR1		    /* Update the SR's parameters	    */
	y=*r2--
	*r2++=y
	*r2=a0
	r0=DQ1		    /**		UPB			   **/
	r3=B1		    /* Set the points for cache.	    */
	r1=DQ16
	i=128
	r2=TR
	do 6 {		    /* Do six times.			    */
	a0=*r1		    /* a0 = DQ				    */
	a1=a0
	y=*r0++		    /* y = DQn				    */
	a1=a1^y	 y=*r3	    /* a1 = Un = DQ ^ DQn		    */
	a1=i		    /* If a1 positive, then Un=0.	    */
	if mi a1=-a1	    /* If a1 negative, then Un=1.	    */
	a0=a0
	if eq a1=a1<<16	    /* a1 = UGBn			    */
	a1=a1+y	 a0=*r3	    /* a1 = UGBn + Bn			    */
	a0=a0>>8	    /* a0 = Bn >> 8			    */
	y=a0
	a1=a1-y	 a0=*r2	    /* a1 = BnP				    */
	a0=a0		    /**		TRIGB			   **/
	if ne a1=a1<<16	    /* If =| 0,	set a1=0.		    */
	*r3++=a1
	}
	r3=DQ16		    /**		FLOATA			   **/
	a0=*r3		    /* a0 = DQ				    */
	a0=a0<<1
	if mi a0=-a0	    /* a0 = |a0|			    */
	if eq a0h=a0h+1
	c1=-16		    /* Set the initial counter to	    */
	a1=a0		    /* find the	EXP of a0.		    */
	do 15 {
	ifc pl a1=a1<<1
	}
	a1=c2
	a1=-a1		    /* a1 = EXP	of DQ			    */
	r2=DQCNT0
	y=tabl1
	a1=a1+y	*r2=a1
	pt=a1
	y=*r2 x=*pt++
	a1=x
	y=a1
	a0=a0&y		    /* a0 contains the magnitude DQ	    */
	r0=DQ0S		    /* Check the sign bit of DQ0.	    */
	a1=*r0
	a1=a1
	if ne a0=-a0	    /* Convert it into 2's complement.	    */
	r2=DQ0
	*r2=a0
	y=*r3		    /**		ADDC			   **/
	r0=SEZ
	a0=*r0
	a0=a0+y		    /* a0 = DQSEZ			    */
	y=0
	move a1=y	    /* If DQSEZ=0, set a1 = 1		    */
	if eq a1h=a1h+1	    /* otherwise, set a1 = 0		    */
	r0=SIGPK
	*r0=a1		    /* Store it	in SIGPK.		    */
	move a1=y
	a0=a0
	if mi a1h=a1h+1	    /* Find the	PK0			    */
	r2=PK0
	*r2=a1
	r0=A1		    /**		UPA2			   **/
	y=*r0
	a0=y *r0	    /* Check A1S = 0 ?			    */
	if mi goto lab26
	y=0x1fff	    /* Yes, compare with 8191.		    */
	a0-y
	if gt a0=y	    /* If a0 > 8191, set a0 = 8191	    */
	a0=a0>>16
	a0=a0<<1
	a0=a0<<1	    /* a0 = a0 >> 14			    */
	goto lab28
lab26:	y=0x0		    /* No, compare with	57345		    */
	yl=*r0
	a0=y
	yl=0xe000
	a1=a0-y		    /* A1 - 57344 >= 0 ?		    */
	a0=a0<<1
	a0=a0<<1
	y=0x1
	yl=0xffff
	a0=a0&y		    /* Yes, a0 = A1 << 2		    */
	yl=0x8004
	a1=a1
	if le a0=y	    /* No, a0 =	24577 << 2		    */
lab28:	y=*r2
	r1=PK1
	a1=y y=*r1 x=*pt++
	a1=a1^y		    /* a1 = PK0	^ PK1 =	PKS1		    */
	if eq a0=-a0	    /* a0 = FA				    */
	y=*r2
	r1=PK2
	a1=y y=*r1 x=*pt++
	a1=a1^y		    /* a1 = PK0	^ PK2 =	PKS2		    */
	y=0x1		    /* Set y = 114688 if a1 < 0		    */
	yl=0xc000
	if ne goto lab31
	y=0x0		    /* otherwise, set y	= 16384		    */
	yl=0x4000
lab31:	a0=a0+y		    /* a0 = UGA2B			    */
	r0=SIGPK
	a0=a0>>1
	a0=a0<<16
	a0=a0>>4
	a0=a0>>1
	a0=a0>>1	    /* a0 = a0 << 9			    */
	y=0x0
	a1=*r0
	a1=a1		    /* Check SIGPK = 0 ?		    */
	if ne a0=y	    /* If not, set a0 =	0.		    */
	r0=A2		    /* a0 = UGA2			    */
	y=*r0
	a0=a0+y		    /* a0 = UGA2 + A2			    */
	a1=y		    /* a1 = A2				    */
	a1=a1>>8
	a1=a1<<1	    /* a1 = A2 >> 7			    */
	y=a1
	a0=a0-y		    /* a0 = A2T				    */
	a1=a0		    /**		LIMC			   **/
	if mi a1=-a1	    /* a1 = |a1|			    */
	y=0x3000	    /* Set y = 12288			    */
	a1-y		    /* |a1| > 12288 ?			    */
	if pl a1=y	    /* Yes, set	|a1| = 12288		    */
	a0=a0
	if mi a1=-a1	    /* a1 = A2P				    */
	r0=A2P
	*r0=a1
	r1=TR		    /**		TRIGB			   **/
	a0=*r1
	a0=a0
	if ne a1=a1<<16	    /* If TR =|	0, set a1 = 0.		    */
	r1=A2
	*r1=a1
	r1=PK1		    /**		UPA1			   **/
	r0=PK0
	a1=*r1++
	y=*r0
	a1=a1^y	 *r1--=a1   /* a1 = PK0	^ PK1 =	PKS		    */
	*r1=y
	y=192
	move a0=y	    /* Set a0 =	192 if PKS = 0		    */
	if ne a0=-a0	    /* otherwise, set a0 = 65344	    */
	r0=SIGPK
	a1=*r0
	a1=a1
	if ne a0=a0<<16	    /* a0=UGA1				    */
	r0=A1
	y=*r0		    /* y = A1				    */
	a0=a0+y		    /* a0 = UGA1 + A1			    */
	a1=y		    /* a1 = A1				    */
	a1=a1>>8	    /* a1 = A1 >> 8			    */
	y=a1
	a0=a0-y		    /* a0 = UGA1 + A1 -	(A1 >> 8)	    */
	r2=A1T
	*r2=a0		    /* a0 = A1T				    */
	y=0x3c00	    /**		LIMD			   **/
	r1=A2P		    /* Set y = 15360			    */
	a1=*r1
	a1=a1-y		    /* a1 = A2P	- 15360			    */
	a1=-a1
	y=a1		    /* y = A1UL				    */
	a1=a0
	if mi a1=-a1	    /* a1 = | A1T |			    */
	a1-y		    /* | A1T | - A1UL			    */
	if pl a1=y
	a0=a0
	if mi a1=-a1	    /* a1 = A1P				    */
	r0=TR		    /**		TRIGB			   **/
	a0=*r0
	a0=a0		    /* Check TR	= 0 ?			    */
	if ne a1=a1<<16	    /* If not, then set	a1 = 0		    */
	r0=A1
	*r0=a1		    /* otherwise, keep a1.		    */
	r0=DQ0
	y=*r0++
	do 6 {
	*r0zp:y		    /* Update the DQn parameters	    */
	}		    /* for the next input data.		    */
	r0=DQ0S
	y=*r0++
	do 6 {
	*r0zp:y		    /* Update the sign bit of DQn	    */
	}		    /* for the next input data.		    */
	r0=DQCNT0
	y=*r0++
	do 6 {
	*r0zp:y		    /* Update the parameters for	    */
	}		    /* the next	input data.		    */

/********************************************************************/
/*								    */
/*			  TONE					    */
/*								    */
/********************************************************************/

totrd1:	r0=A2P		    /**		TONE  totrde.s		   **/
	a0=*r0
	y=0x2e00	    /* y = 11776			    */
	a0=a0+y		    /* a2(k) + 0.71875			    */
	y=0
	move a1=y
	if mi a1h=a1h+1	    /* a1 = TDP				    */
	r0=TDP
	*r0=a1
	r3=TR		    /**		TRIGB			   **/
	a0=*r3
	a0=a0		    /* Check TR	= 0 ?			    */
	*r1=a1
adspc1:	r0=I		    /**		FUNCTF	adspcl.s	   **/
	a0=*r0		    /* a0 = I				    */
	y=FI
	a0=a0+y		    /* Use the table looking up	to	    */
	r2=DML		    /* find the	FI (scaled) value.	    */
	pt=a0
	y=*r2 x=*pt++
	a0=x		    /* a0 = FI				    */
	a1=a0<<1	    /**		FILTB			   **/
	a1=a1<<1	    /* a1 = FI << 11			    */
	a1=a1-y		    /* a1 = (FI<<11) - DML = DIF	    */
	a1=a1<<1
	a1=a1>>8	    /* a1 = DIF	>> 7 = DIFSX		    */
	a1=a1+y		    /* a1 = DIFSX + DML			    */
	y=0x3fff	    /* Set y = 16383			    */
	r1=DMS
	a1=a1&y	y=*r1
	a0=a0-y	*r2=a1	    /**		FILTA			   **/
	a0=a0>>1
	a0=a0<<4
	a0=a0>>8	    /* a0 = DIFSX = DIF	>> 5		    */
	a0=a0+y		    /* a0 = DIFSX + DMS			    */
	y=0xfff		    /* Set y = 4095			    */
	a0=a0&y	y=*r2
	*r1=a0		    /* a0 = DMSP			    */
	a0=a0<<1	    /**		SUBTC			   **/
	a0=a0<<1	    /* a0 = DMSP << 2			    */
	a0=a0-y	 a1=*r2	    /* a0 = (DMSP<<2) -	DMLP = DIF	    */
	if mi a0=-a0	    /* a0 = | DIF | = DIFM		    */
	a1=a1>>4
	a1=a1<<1	    /* a1 = DTHR = DMLP	>> 3		    */
	move y=a1
	r3=Y
	a0=a0-y	 a1=*r3	    /* a0 = DIFM - DTHR			    */
	r3=TDP
	a0=*r3
	y=1
	if pl a0=y
	y=1536		    /* Set y = 1536			    */
	a1=a1-y		    /* a1 = Y -	1536			    */
	y=1
	if mi a0=y	    /* a0 = AX				    */
	a0=a0<<8	    /**		FILTC			   **/
	a0=a0<<1	    /* a0 = AX << 9			    */
	r3=AP
	y=*r3		    /* y = AP				    */
	a0=a0-y		    /* a0 = (AX<<9) - AP = DIF		    */
	a0=a0>>4	    /* a0 = DIFSX			    */
	a0=a0+y		    /* a0 = DIFSX + AP			    */
	y=0x3ff
	r2=TR		    /**		TRIGA			   **/
	a0=a0&y	y=*r2	    /* a0 = APP				    */
	a1=y *r2
	y=0x100		    /* Set y = 256			    */
	if ne a0=y
	*r3=a0		    /* a0 = APR				    */

/********************************************************************/
/*								    */
/*			  FUNCTW				    */
/*								    */
/********************************************************************/

qsfad1:	r0=I		    /**		FUNCTW	qsfadp.s	   **/
	a0=*r0
	y=WI
	a0=a0+y		    /* a0 points to WI			    */
	pt=a0
	y=*r0 x=*pt++	    /* x = WI				    */
	a0=x		    /**		FILTD			   **/
	a0=a0>>8
	a0=a0>>4
	a0=a0<<1	    /* a0l = WI	<< 5			    */
	r2=Y
	y=0x0
	yl=*r2		    /* y = Y				    */
	a0=a0-y		    /* a0 = DIF				    */
	a0=a0>>4
	a0=a0>>1	    /* a0 = DIFSX = DIF	>> 5		    */
	a0=a0+y		    /* a0 = Y +	DIFSX			    */
	a0=a0<<16
	y=0x1fff	    /* Set y = 8191			    */
	a0=a0&y		    /* a0 = YUT				    */
	y=544		    /**		LIMB			   **/
	a0-y		    /* If negative, GELL = 1		    */
	if mi a0=y
	y=5120		    /* Set y = 5120			    */
	a0-y		    /* If positive, GEUL = 0		    */
	if pl a0=y	    /* a0 = YUP				    */
	r3=YU
	*r3=a0		    /* Store it.			    */
	y=0
	yl=a0
	r1=YL		    /**		FILTE			   **/
	a0=*r1++
	a0l=*r1--	    /* a0 = YL				    */
	a0=-a0		    /* a0 = -YL				    */
	a0=a0>>4
	a0=a0>>1
	a0=a0>>1	    /* a0 = ( -YL ) >> 6		    */
	a0=a0+y		    /* a0 = DIF	= DIFSX			    */
	y=*r1++
	yl=*r1--	    /* y = YL				    */
	a0=a0+y		    /* a0 = YL + DIFSX			    */
	y=0x7
	yl=0xffff
	a0=a0&y		    /* a0 = YLP				    */
	*r1++=a0
	*r1=a0l		    /* Store it.			    */
	goto loop	    /* Go back to execute the next	    */
			    /* sampling	data.			    */

/********************************************************************/
/*								    */
/*		  End of Program				    */
/*								    */
/********************************************************************/

posx:	int	   0x9
	int	   0xa
	int	   0xb
	int	   0xc
	int	   0xd
	int	   0xe
	int	   0xf
	int	   0x7
	int	   0x9
negx:	int	   0x6
	int	   0x5
	int	   0x4
	int	   0x3
	int	   0x2
	int	   0x1
	int	   0x0
	int	   0x7
	int	   0x6
WI:	int	   0xfff4	/* -12 = 4084 for 12 bits	    */
	int	   18
	int	   41
	int	   64
	int	   112
	int	   198
	int	   355
	int	   1122
	int	   1122
	int	   355
	int	   198
	int	   112
	int	   64
	int	   41
	int	   18
	int	   0xfff4
FI:	int	   0
	int	   0
	int	   0
	int	   0x200
	int	   0x200
	int	   0x200
	int	   0x600
	int	   0xe00
	int	   0xe00
	int	   0x600
	int	   0x200
	int	   0x200
	int	   0x200
	int	   0
	int	   0
	int	   0
pos:	int	   0x01
	int	   0x02
	int	   0x03
	int	   0x04
	int	   0x05
	int	   0x06
	int	   0x07
	int	   0x0f
	int	   0x01
neg:	int	   0x0e
	int	   0x0d
	int	   0x0c
	int	   0x0b
	int	   0x0a
	int	   0x09
	int	   0x08
	int	   0x0f
	int	   0x0e
dln:	int	   79
	int	   177
	int	   245
	int	   299
	int	   348
	int	   399
	int	   2047
	int	   3971
	int	   4095
dqln:	int	   2048
	int	   4
	int	   135
	int	   213
	int	   273
	int	   323
	int	   373
	int	   425
tabl1:	int	   0x0
	int	   0x3f
	int	   0x3f
	int	   0x3f
	int	   0x3f
	int	   0x3f
	int	   0x3f
	int	   0x7e
	int	   0xfc
	int	   0x1f8
	int	   0x3f0
	int	   0x7e0
	int	   0xfc0
	int	   0x1f80
	int	   0x3f00
	int	   0x7e00
	int	   0x7e00
tabl2:	28*int	      0x0
	int	   0x3
	int	   0x7fff
	int	   0x6
	int	   0x7fff
	int	   0xc
	int	   0x7fff
	int	   0x18
	int	   0x7fff
	int	   0x30
	int	   0x7fff
	int	   0x60
	int	   0x7fff
	int	   0xc0
	int	   0x7fff
	int	   0x180
	int	   0x7fff
	int	   0x3
	int	   0x6fff
	int	   0x6
	int	   0x6ffe
	int	   0xc
	int	   0x6ffc
	int	   0x18
	int	   0x6ff8
	int	   0x30
	int	   0x6ff0
	int	   0x60
	int	   0x6fe0
	int	   0xc0
	int	   0x6fc0
	int	   0x180
	int	   0x6f80
	int	   0x300
	int	   0x6f00

.ram
SD:	int
DLN:	int
S:	int
SL:	int
D:	int
temp:	int
temp1:	int
Y:	int
YSH2:	int
I:	int
R:	int
YL:	int
	int
YU:	int
AL:	int
TDP:	int
AP:	int
DML:	int
DMS:	int
DQ0:	int
DQ1:	int
DQ2:	int
DQ3:	int
DQ4:	int
DQ5:	int
DQ6:	int
SR2:	int
SR1:	int
B1:	int
B2:	int
B3:	int
B4:	int
B5:	int
B6:	int
A2:	int
A1:	int
	int
WB1:	int
WB2:	int
WB3:	int
WB4:	int
WB5:	int
WB6:	int
WA2:	int
WA1:	int
PK1:	int
PK2:	int
PK0:	int
A2P:	int
SE:	int
SEZ:	int
DQ16:	int	       /* The TC of DQ				    */
TR:	int
A1T:	int
SIGPK:	int
SR:	int
TD:	int
DQ0S:	int
DQ1S:	int
DQ2S:	int
DQ3S:	int
DQ4S:	int
DQ5S:	int
DQ6S:	int
SR2S:	int
SR1S:	int
DQCNT0:	int
DQCNT1:	int
DQCNT2:	int
DQCNT3:	int
DQCNT4:	int
DQCNT5:	int
DQCNT6:	int
SRCNT2:	int
SRCNT1:	int
.endram

